@@ -1,5 +1,5 @@
 /* config.c
- * Copyright (C) 2004, 2005 Sylvain Cresto <sylvain.cresto@tiscali.fr>
+ * Copyright (C) 2004, 2005 Sylvain Cresto <scresto@gmail.com>
  *
  * This file is part of graveman!
  *
@@ -46,25 +46,31 @@ TProgRequis Glisteprogrequis[] = {
   { CONF_STRING, "dvd+rw-formatpara", "", CONF_SAVEALWAYS },
   { CONF_PROG, "growisofs", "", CONF_SAVEALWAYS },
   { CONF_STRING, "growisofspara", "", CONF_SAVEALWAYS },
+  { CONF_PROG, "cdrdao", "", CONF_SAVEALWAYS },
+  { CONF_STRING, "cdrdaopara", "", CONF_SAVEALWAYS },
+  { CONF_PROG, "flac", "", CONF_SAVEALWAYS },
+  { CONF_STRING, "flacpara", "", CONF_SAVEALWAYS },
   { CONF_DIR, "tmpdir", "/tmp", CONF_SAVEALWAYS },
   { CONF_STRING, "overburn", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "autoblank", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "fastblank", "1", CONF_SAVEALWAYS },
+  { CONF_STRING, "charsetdata", "DEFAULT", CONF_SAVEALWAYS },
   { CONF_STRING, "eject", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "iconsize", "48", CONF_SAVEALWAYS },
   { CONF_STRING, "statusbar", "1", CONF_SAVEALWAYS },
+  { CONF_STRING, "tooltips", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "saveconfig", "1", CONF_SAVEALWAYS },
 
   /* onglet audio */
-  { CONF_STRING, "dstaudiocombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstaudiocombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstaudiospeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "audiodao", "1", CONF_SAVEONEXIT + CONF_CHECK },
-  { CONF_STRING, "audionotfix", "1", CONF_SAVEONEXIT + CONF_CHECK },
-  { CONF_STRING, "audiosimul", "1", CONF_SAVEONEXIT + CONF_CHECK },
+  { CONF_STRING, "audionotfix", "", CONF_SAVEONEXIT + CONF_CHECK },
+  { CONF_STRING, "audiosimul", "", CONF_SAVEONEXIT + CONF_CHECK },
   { CONF_STRING, "nbrcdaudio", "1", CONF_SAVEONEXIT + CONF_SPIN },
   
   /* onglet data */
-  { CONF_STRING, "dstdatacombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstdatacombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstdataspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "nbrcddata", "1", CONF_SAVEONEXIT + CONF_SPIN },
   { CONF_STRING, "dataformat", "", CONF_SAVEONEXIT + CONF_COMBO },
@@ -76,19 +82,30 @@ TProgRequis Glisteprogrequis[] = {
   { CONF_STRING, "datanotfix", "", CONF_SAVEONEXIT + CONF_CHECK },
   { CONF_STRING, "datasimul", "", CONF_SAVEONEXIT + CONF_CHECK },
 
+  /* onglet dvd data */
+  { CONF_STRING, "dstdvddatacombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
+  { CONF_STRING, "dstdvddataspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "nbrdvddata", "1", CONF_SAVEONEXIT + CONF_SPIN },
+  { CONF_STRING, "dvddatanotfix", "", CONF_SAVEONEXIT + CONF_CHECK },
+  { CONF_STRING, "dvddatasimul", "", CONF_SAVEONEXIT + CONF_CHECK },
+
+
  /* onglet copier */
-  { CONF_STRING, "srccopycombo", "", CONF_SAVEONEXIT + CONF_COMBO },
-  { CONF_STRING, "dstcopycombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "srccopycombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
+  { CONF_STRING, "dstcopycombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstcopyspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "nbrcdcopy", "1", CONF_SAVEONEXIT + CONF_SPIN },
-  { CONF_STRING, "copysimul", "1", CONF_SAVEONEXIT + CONF_CHECK },
+  { CONF_STRING, "copysimul", "", CONF_SAVEONEXIT + CONF_CHECK },
 
   /* onglet autres */
-  { CONF_STRING, "dstothercombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstothercombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   
 #if 0
   { CONF_STRING, "extended", FALSE, "1" },
 #endif
+  { CONF_STRING, "version", "1", CONF_SAVEALWAYS },
+  { CONF_STRING, "scan_drives", "0", CONF_SAVEALWAYS + CONF_PERSIST },
+  { CONF_STRING, "theme", "default", CONF_SAVEALWAYS },
   { CONF_IGNORE, NULL, NULL, 0 },
 };
 
@@ -102,23 +119,6 @@ TProgRequis *getentrydef(gchar *Aentry)
   return NULL;
 }
 
-/* recuperation du nom de fichier ou ecrire */
-gchar *get_config_file_name()
-{
-  const gchar *LHome = g_getenv("HOME");
-  gchar *Ldir = g_strdup_printf("%s/.%s", LHome, PACKAGE);
-  gchar *Lfile = g_strdup_printf("%s/.%s/%s.conf", LHome, PACKAGE, PACKAGE);
-
-  /* tentative de creation du repertoire si il n'existe pas deja */
-  if (g_file_test(Ldir, G_FILE_TEST_EXISTS) == FALSE) {
-    mkdir(Ldir, 00750);
-  }
-  g_free(Ldir);
-
-  return (Lfile);
-}
-
-
 /* callback ecriture fichier de conf */
 void foreachconfig(gpointer Akey, gpointer Avalue, gpointer Adata)
 {
@@ -139,6 +139,17 @@ void foreachmedia(gpointer Amedia, gpointer Adata)
   fprintf(Lfic, "audiolabel=%s\n", Amediaentry->labelaudio);
 }
 
+/* callback ecriture d'un charset */
+void foreachcharsets(gpointer Acharset, gpointer Adata)
+{
+  FILE *Lfic = (FILE *) Adata;
+  Tdata *Lcharset = (Tdata *) Acharset;
+
+  fprintf(Lfic, "[charsets]\n");
+  fprintf(Lfic, "code=%s\n", Lcharset->data);
+  fprintf(Lfic, "label=%s\n", Lcharset->label);
+}
+
 /* callback ecriture d'un lecteur */
 void foreachdrives(gpointer Adrive, gpointer Adata)
 {
@@ -154,36 +165,57 @@ void foreachdrives(gpointer Adrive, gpointer Adata)
   fprintf(Lfic, "name=%s\n", Ldrive->name);  
   fprintf(Lfic, "type=%d\n", Ldrive->type);
   fprintf(Lfic, "vitesse=%d\n", Ldrive->vitesse);
+  fprintf(Lfic, "vitessedvd=%d\n", Ldrive->vitessedvd);
   fprintf(Lfic, "manual=%d\n", Ldrive->manual == TRUE ? 1 : 0);
 }
 
+gchar *dump_file_list(char *Abuf, gint Asize)
+{
+  GSList *Llistconfig = get_config_path();
+  GSList *Lcurlist;
+
+  memset(Abuf, 0, Asize);
+
+  for (Lcurlist = Llistconfig; Lcurlist; Lcurlist = g_slist_next(Lcurlist)) {
+    if (*Abuf) strncat(Abuf, "\n", Asize-strlen(Abuf)-5);
+    strncat(Abuf, Lcurlist->data, Asize-strlen(Abuf)-5);
+  }
+
+  g_slist_free(Llistconfig);
+
+  return Abuf;
+}
+
 /* ecriture du fichier de configuration */
-gint write_conf(gchar *AFichier, GError **Aerror)
+gboolean write_conf(GError **Aerror)
 {
   FILE *Lfic;
   gchar *Ltmp;
+  
+  if (!get_config_file(W_OK)) return FALSE;
 
-  if (!(Lfic=fopen(AFichier, "w"))) {
-    Ltmp = g_strdup_printf(_("Cannot create %s: %s"), AFichier, g_strerror(errno));
+  if (!(Lfic=fopen(Gfileconf, "w"))) {
+    Ltmp = g_strdup_printf(_("Cannot create %s: %s"), Gfileconf, g_strerror(errno));
     g_set_error(Aerror, G_FILE_ERROR, g_file_error_from_errno(errno), Ltmp, g_strerror(errno)); 
     g_warning("%s", Ltmp);
     g_free(Ltmp);
     
-    return EXIT_FAILURE;
+    return FALSE;
   }
 
   fprintf(Lfic, "[general]\n");
+  fprintf(Lfic, "confversion=%s\n", VERSION);
   g_hash_table_foreach(Gconfigprog, foreachconfig, (gpointer *)(Lfic));
-
-/*  g_slist_foreach(Gmedialist, foreachmedia, (gpointer *)(fic)); */
   
   /* ecriture des lecteurs */
-
   g_slist_foreach(Glistdrives, foreachdrives, (gpointer *)(Lfic));
 
+  /* ecriture des charsets */
+  g_slist_foreach(Gcharsetlist, foreachcharsets, (gpointer *)(Lfic));
+
   fclose(Lfic);
 
-  return EXIT_SUCCESS;
+  return TRUE;
 }
 
 /* initialisation de la configuration et des valeurs par default */
@@ -199,6 +231,8 @@ void config_init()
   conf_store_value("iconsize", "48");
   conf_store_value("statusbar", "1");
   conf_store_value("saveconfig", "1");
+  conf_store_value("tooltips", "1");
+  conf_store_value("theme", "default");
   if (s) {
     Gfifosize = atoi(s);
     if (Gfifosize<=0) Gfifosize=6;
@@ -206,22 +240,134 @@ void config_init()
 }
 
 /* on regarde si un fichier de configuration existe 
- * dans /usr/share/graveman/graveman.rc ou ~/.graveman/graveman.rc */
-gint is_config_valid()
+ * tel que definie par http://standards.freedesktop.org/basedir-spec/latest/ */
+
+/* construction de la liste des PATH dans lequel on va chercher la configuration */
+GSList *get_config_path()
 {
-  const gchar *LHome = g_getenv("HOME");
-  gchar LUserConf[1024], LGlobalConf[1024];
+  GSList *Lnewlist = NULL;
+  const gchar *Lenv;
+  gchar *Lpath;
+  gchar **Llistpath;
+  gint i;
 
-  bzero(&LUserConf, sizeof(LUserConf));
-  bzero(&LGlobalConf, sizeof(LGlobalConf));
+  /* parametre -c sur la ligne de commande */
+  if ((Lpath = param_get_string("config_file"))) {
+    Llistpath = g_strsplit(Lpath, ":", 0);
 
-  g_snprintf(LUserConf, sizeof(LUserConf)-1, "%s/.%s/%s.conf", LHome, PACKAGE, PACKAGE);
-  g_snprintf(LGlobalConf, sizeof(LGlobalConf)-1, "%s/%s/%s.conf", PACKAGE_DATA_DIR, PACKAGE, PACKAGE);
+    for (i=0; Llistpath[i]; i++) {
+      if (!*Llistpath[i]) continue; /* on ignore les entrees vides */
+      Lnewlist = g_slist_append(Lnewlist, g_strdup_printf(Llistpath[i]));
+    }
 
-  if (*LUserConf && access(LUserConf, F_OK + R_OK + W_OK)==0) {
-    strcpy(Gfileconf, LUserConf);
-  } else if (*LGlobalConf && access(LGlobalConf, F_OK + R_OK + W_OK)==0) {
-    strcpy(Gfileconf, LGlobalConf);
+    g_strfreev(Llistpath);
+  }
+  
+  if ((Lenv=g_getenv("XDG_CONFIG_HOME"))) {
+    Lpath = g_strdup_printf("%s/%s/%s.conf", Lenv, PACKAGE, PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  } else if ((Lenv=g_getenv("HOME"))) {
+    Lpath = g_strdup_printf("%s/.config/%s/%s.conf", Lenv, PACKAGE, PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  }
+
+  if ((Lenv = g_getenv("XDG_CONFIG_DIRS"))) {
+    Llistpath = g_strsplit(Lenv, ":", 0);
+
+    for (i=0; Llistpath[i]; i++) {
+      if (!*Llistpath[i]) continue; /* on ignore les entrees vides */
+      Lpath = g_strdup_printf("%s/%s/%s.conf", Llistpath[i], PACKAGE, PACKAGE);
+      Lnewlist = g_slist_append(Lnewlist, Lpath);
+    }
+
+    g_strfreev(Llistpath);
+
+  } else {
+    Lpath = g_strdup_printf("/etc/xdg/%s/%s.conf", PACKAGE, PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  }
+
+  return Lnewlist;
+}
+
+/* on regarde si un repertoire de donne existe
+ * tel que definie par http://standards.freedesktop.org/basedir-spec/latest/
+ * on regarde de + en dernier dans le DATADIR definie a la compilation
+ */
+
+/* construction de la liste des PATH dans lequel on va chercher la configuration */
+GSList *get_data_path()
+{
+  GSList *Lnewlist = NULL;
+  const gchar *Lenv;
+  gchar *Lpath;
+  gchar **Llistpath;
+  gint i;
+
+  /* parametre -c sur la ligne de commande */
+  if ((Lpath = param_get_string("data_file"))) {
+    Llistpath = g_strsplit(Lpath, ":", 0);
+
+    for (i=0; Llistpath[i]; i++) {
+      if (!*Llistpath[i]) continue; /* on ignore les entrees vides */
+      Lnewlist = g_slist_append(Lnewlist, g_strdup_printf(Llistpath[i]));
+    }
+
+    g_strfreev(Llistpath);
+  }
+
+  if ((Lenv=g_getenv("XDG_DATA_HOME"))) {
+    Lpath = g_strdup_printf("%s/%s", Lenv, PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  } else if ((Lenv=g_getenv("HOME"))) {
+    Lpath = g_strdup_printf("%s/.local/share/%s", Lenv, PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  }
+
+  if ((Lenv = g_getenv("XDG_DATA_DIRS"))) {
+    Llistpath = g_strsplit(Lenv, ":", 0);
+
+    for (i=0; Llistpath[i]; i++) {
+      Lpath = g_strdup_printf("%s/%s/%s.conf", Llistpath[i], PACKAGE, PACKAGE);
+      Lnewlist = g_slist_append(Lnewlist, Lpath);
+    }
+
+    g_strfreev(Llistpath);
+  } else {
+    Lpath = g_strdup_printf("/usr/local/share/%s", PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+    Lpath = g_strdup_printf("/usr/share/%s", PACKAGE);
+    Lnewlist = g_slist_append(Lnewlist, Lpath);
+  }
+
+  return Lnewlist;
+}
+
+gboolean get_config_file(gint Lmode)
+{
+  GSList *Llistconfig = get_config_path();
+  GSList *Lcurlist;
+  gboolean Ltrouve = FALSE;
+  gchar Lonlyname[1024];
+  gchar *s;
+  
+  *Gfileconf = 0; 
+
+  for (Lcurlist = Llistconfig; Lcurlist; Lcurlist = g_slist_next(Lcurlist)) {
+    if (W_OK == Lmode) {
+      g_strlcpy(Lonlyname, (gchar *)Lcurlist->data, sizeof(Lonlyname)-1);      
+      if (!(s=strrchr(Lonlyname, '/'))) continue;
+      *s=0;
+      if (FALSE == sc_mkdir(Lonlyname, 00750)) continue;
+
+      Ltrouve = TRUE;
+    } else if (access((gchar *)Lcurlist->data, F_OK)==0) {
+      Ltrouve = TRUE;
+    }
+    if (Ltrouve) {
+      g_strlcpy(Gfileconf, (gchar *) Lcurlist->data, sizeof(Gfileconf)-1);
+      break;
+    }
   }
 
 #ifdef DEBUG
@@ -232,52 +378,12 @@ gint is_config_valid()
   }
 #endif
 
-  return (*Gfileconf);
-}
+  if (!Ltrouve)
+    g_strlcpy(Gfileconf, (gchar *) g_slist_nth_data(Llistconfig, 0), sizeof(Gfileconf)-1);
 
-/* on cherche un programme dans le PATH */
-gchar *trouveUnProg(gchar *Anomprog)
-{
-  struct stat Lficinfo;
-  gchar Ltest[2048], *Lpath, *s, *p;
-  gchar *Lrespath = NULL;
+  g_slist_free(Llistconfig);
 
-  Lpath = g_strdup(g_getenv("PATH"));
-
-  p = Lpath;
-  while (((s = strchr(p, ':'))) || p) {
-    if (s) *(s++)=0;
-
-    g_snprintf(Ltest, sizeof(Ltest)-1, "%s/%s", p, Anomprog);
-    if (0==stat(Ltest, &Lficinfo) && S_ISREG(Lficinfo.st_mode)) {
-      Lrespath = strdup(Ltest);
-      break;
-    } 
-    if (!s) break;
-    p=s;
-  }
-
-  g_free(Lpath);
-
-#ifdef DEBUG
-  if (Lrespath) {
-    _DEB("on a trouve [%s]\n", Lrespath);
-  }
-#endif
-
-  return Lrespath;
-}
-
-/* le repertoire est-il valide ? sinon renvoi celui par defaut */
-gchar *trouveUnPath(gchar *Anompath, gchar *Adefaultpath)
-{
-  struct stat Lficinfo;
-
-  if (0==stat(Anompath, &Lficinfo) && S_ISDIR(Lficinfo.st_mode)) {
-    return strdup(Anompath);
-  }
-
-  return strdup(Adefaultpath);
+  return Ltrouve;
 }
 
 /* sauve une cle dans le hash de configuration */
@@ -331,6 +437,19 @@ gint config_find_media_callback(gconstpointer Acompsize, gconstpointer Acompvers
   return -1;
 }
 
+/* ajout d'un charset a  la liste des charsets */
+gboolean config_append_charset(gchar *Acode, gchar *Alabel)
+{
+  Tdata *Lnewcharset = g_malloc(sizeof(Tdata));
+
+  Lnewcharset->data = g_strdup(Acode);
+  Lnewcharset->label = g_strdup(Alabel && *Alabel ? Alabel : Acode);
+
+  Gcharsetlist = g_slist_append(Gcharsetlist, Lnewcharset);
+
+  return TRUE;
+}
+
 /* ajout d'un media a la liste des media */
 gboolean config_append_media(gulong Alen, gchar *Alabeldata, gchar *Alabelaudio)
 {
@@ -352,14 +471,14 @@ gboolean config_append_media(gulong Alen, gchar *Alabeldata, gchar *Alabelaudio)
 
 /* savegarde de la valeur de certains champs du formulaire lorsque
  * l'option "save on exit" est active */
-void manage_saveonexit_value(GHashTable *Ahash)
+void manage_saveonexit_value(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   GtkWidget *Lwidget;
 
   for (Lcurentry = Glisteprogrequis; Lcurentry->entry; Lcurentry++) {
     if (!(Lcurentry->level & CONF_SAVEONEXIT)) continue;
-    Lwidget = g_hash_table_lookup(Ahash, Lcurentry->entry);
+    Lwidget = sc_grave_get_widget(Ag, Lcurentry->entry);
 
     if (!GTK_IS_WIDGET(Lwidget)) {
       g_warning("Error %s is missing!", Lcurentry->entry);
@@ -369,6 +488,10 @@ void manage_saveonexit_value(GHashTable *Ahash)
         
     if (Lcurentry->level & CONF_COMBO) {
       conf_store_value(Lcurentry->entry, get_combo_value(Lwidget));
+    } else if (Lcurentry->level & CONF_DEVICE) {
+      Tdriveinfo *Ldevice = get_combo_value(Lwidget);
+      if (Ldevice != NULL)
+        conf_store_value(Lcurentry->entry, DRIVE_DEV(Ldevice));
     } else if (Lcurentry->level & CONF_SPIN) {
       conf_store_int(Lcurentry->entry, gtk_spin_button_get_value(GTK_SPIN_BUTTON(Lwidget)));
     } else if (Lcurentry->level & CONF_CHECK) {
@@ -378,37 +501,61 @@ void manage_saveonexit_value(GHashTable *Ahash)
 }
 
 /* gestion de la configuration (creation/chargement/sauvegarde...) */
-gint manage_config(GHashTable *Ahash, gshort Aop, GError **Aerror)
+gboolean manage_config(Tgrave *Ag, gshort Aop, GError **Aerror)
 {
   TProgRequis *Lcurprog;
   gchar *Lprog;
-  GtkWidget *Lwindow1;
-  Lwindow1 = g_hash_table_lookup(Ahash, "topwindow");
+  gboolean Lstatus = TRUE;
+  GtkWidget *Lwindow1 = sc_grave_get_widget(Ag, "topwindow");
 
   if (Aop & UPDATE_SAVEONEXIT_VALUE) {
     /* mise a jour des valeurs a sauvegarder lorsque l'on quitte */
-    manage_saveonexit_value(Ahash);
+    manage_saveonexit_value(Ag);
+  }
+
+  if (Aop & CONF_INIT_DEFAULT) {
+    /* initialsiation des valeurs par defaut */
+    for (Lcurprog = Glisteprogrequis; Lcurprog->entry; Lcurprog++) {
+      if (Lcurprog->defaultvalue && *Lcurprog->defaultvalue) {
+        Lprog = g_strdup(Lcurprog->defaultvalue);
+
+        conf_store_value(Lcurprog->entry, Lprog);
+      }
+    }
   }
 
   if (Aop & SEARCH_PROG) {
     /* on recherche les programmes externes */
     for (Lcurprog = Glisteprogrequis; Lcurprog->entry; Lcurprog++) {
-      if (Lcurprog->type == CONF_PROG) {
-        Lprog = trouveUnProg(Lcurprog->entry);
-      } else {
-        Lprog = g_strdup(Lcurprog->defaultvalue);
-      }
+      if (Lcurprog->type &~ CONF_PROG) continue;
+
+      Lprog = g_find_program_in_path(Lcurprog->entry);
 
       conf_store_value(Lcurprog->entry, Lprog);
 #ifdef DEBUG
-      if (! Lprog) g_warning("ERREUR LE PROG [%s] EST INDISPENSABLE ET NON TROUVE !\n", Lcurprog->entry);
+      if (! Lprog || !*Lprog) g_warning("ERREUR LE PROG [%s] EST INDISPENSABLE ET NON TROUVE !\n", Lcurprog->entry);
 #endif
     }
+    
+    /* pour mkisofs on regarde qu'il y ai bien le support iso version 2 */
+    conf_store_value("mkisofs_support_isov2", _BOOLEAN_CONF(check_mkisofs()));
+
+    /* pour sox on regarde si il a bien le support ogg / mp3 */
+    check_sox();
+    conf_store_value("sox_support_mp3", _BOOLEAN_CONF(GsupportMp3));
+    conf_store_value("sox_support_ogg", _BOOLEAN_CONF(GsupportOgg));
+    
+    GsupportFlac = *conf_get_string("flac");
   }
 
   if (Aop & SEARCH_LECTEUR) {
     /* maintenant scan des lecteurs */
-    cherchelecteur(NULL, Ahash);
+    cherchelecteur(NULL, Ag);
+  }
+
+  if (Aop & SEARCH_CHARSETS) {
+    /* scan des charsets definie dans mkisofs */
+    cherchecharset(NULL, Ag);
   }
 
   if (Aop & UPDATE_SIZE && GTK_IS_WIDGET(Lwindow1)) {
@@ -422,14 +569,25 @@ gint manage_config(GHashTable *Ahash, gshort Aop, GError **Aerror)
     conf_store_int("height", Lheight);
   }
 
-  if (Aop & WRITE_CONFIG) {
-    gchar *Lconffile = get_config_file_name();
-    gint Lwrstat = write_conf(Lconffile, Aerror);
-    g_free(Lconffile);
-    return Lwrstat;
+  if (Aop & WRITE_CONFIG) {    
+    Lstatus = write_conf(Aerror);
+    if (!Lstatus) {
+      gchar Lbuf[_BUF_SIZE];
+      gchar *Lfullmsg = g_strdup_printf(_("Cannot save configuration file !\nI have tried on those files:\n%s"),
+          dump_file_list(Lbuf, sizeof(Lbuf)-1));
+      GtkWidget *Ldialog = gtk_message_dialog_new(GTK_IS_WIDGET(Lwindow1) ? GTK_WINDOW(Lwindow1) : NULL,
+         GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+         Lfullmsg);
+      
+      g_warning("%s", Lfullmsg);
+
+      gtk_dialog_run(GTK_DIALOG(Ldialog));
+      gtk_widget_destroy(Ldialog);
+      g_free(Lfullmsg);
+    }
   }
 
-  return 1;
+  return Lstatus;
 }
 
 /* renvoi une valeur du fichier de configuration */
@@ -492,11 +650,11 @@ gboolean read_config(GError **Aerror)
   gchar *Lkey, *Lvalue;
   gchar **Llignes;
   gchar *Luneligne;
-  gchar Lemplacement[50] = "";
+  gchar Lemplacement[_BUF_SIZE] = "";
   gint i;
   gulong Lmedialen = 0;
-  gchar Lmedialabeldata[50];
-  gchar Lmedialabelaudio[50];
+  gchar Lmedialabeldata[_BUF_SIZE] = "", Lcharsetcode[_BUF_SIZE] = "";
+  gchar Lmedialabelaudio[_BUF_SIZE] = "", Lcharsetlabel[_BUF_SIZE] = "";
   Tdriveinfo *Ldriveinfo = NULL;
 
   if (!g_file_get_contents(Gfileconf, &Lcontents, NULL, Aerror)) {
@@ -532,6 +690,23 @@ gboolean read_config(GError **Aerror)
       Lkey = Luneligne;
 
       g_hash_table_insert(Gconfigprog, g_strdup(Lkey), g_strdup(Lvalue));
+    } else if (!strcmp(Lemplacement, "[charsets]")) {
+      /* section charsets */
+      Lvalue = strchr(Luneligne, '=');
+      if (Lvalue) { *(Lvalue++)=0; } else { continue; }
+      Lkey = Luneligne;
+
+      if (!strcmp(Lkey, "code")) {
+        g_strlcpy(Lcharsetcode, Lvalue, sizeof(Lcharsetcode)-1);
+      } else if (!strcmp(Lkey, "label")) {
+        g_strlcpy(Lcharsetlabel, Lvalue, sizeof(Lcharsetlabel)-1);
+      }
+
+      if (*Lcharsetcode && *Lcharsetlabel) {
+        config_append_charset(Lcharsetcode, Lcharsetlabel);
+
+        *Lcharsetcode = *Lcharsetlabel = 0;
+      }
     } else if (!strcmp(Lemplacement, "[media]")) {
       /* section liste des medias */
       Lvalue = strchr(Luneligne, '=');
@@ -587,40 +762,75 @@ gboolean read_config(GError **Aerror)
   g_strfreev(Llignes);
   g_free(Lcontents);
   clean_config();
+
+  GsupportMp3 = conf_get_boolean("sox_support_mp3");
+  GsupportOgg = conf_get_boolean("sox_support_ogg");
+  GsupportFlac = *conf_get_string("flac");
+
+  if (!test_version()) return FALSE;  
   
   return TRUE;
 }
 
 void maj_foreach_prog(gpointer Akey, gpointer Avalue, gpointer Adata)
 {
-  GHashTable *Lhash = (GHashTable *) Adata;
-  GtkWidget *Lobj = g_hash_table_lookup(Lhash, Akey);
-  gchar *Lname;
+  Tgrave *Lg = (Tgrave *) Adata;
+  GtkWidget *Lobj = sc_grave_get_widget(Lg, Akey);
+  gchar *Lname, *Lconfvalue;
   GtkEntry *Lobjentry;
   GtkWidget *Lobjlbl;
+  GtkWidget *Lobjtips;
+  gchar Ltipstxt[_BUF_SIZE];
+  
+  if (!GTK_IS_WIDGET(Lobj)) return;
 
-  if (!Lobj) return;
   Lname = g_strdup_printf("%s%s", gtk_widget_get_name(Lobj), "para");
-  Lobjentry = GTK_ENTRY(g_hash_table_lookup(Lhash, Lname));
-  gtk_entry_set_text(Lobjentry, *(gchar *)conf_get_string(Lname) ? conf_get_string(Lname) : "");
+  Lobjentry = GTK_ENTRY(sc_grave_get_widget(Lg, Lname));
+
+  Lconfvalue = conf_get_string(Lname);
+  gtk_entry_set_text(Lobjentry, Lconfvalue ? Lconfvalue : "");
   g_free(Lname);
 
-  Lname = g_strdup_printf("%s%s", gtk_widget_get_name(Lobj), "lbl");
-  Lobjlbl = g_hash_table_lookup(Lhash, Lname);
-  g_free(Lname);
+  Lobjlbl = sc_grave_get_widgetv(Lg, "%s%s", gtk_widget_get_name(Lobj), "lbl");
+  Lobjtips = sc_grave_get_widgetv(Lg, "%s%s", gtk_widget_get_name(Lobj), "tips");
       
   if (*(gchar *)Avalue) {
-    gtk_image_set_from_stock(GTK_IMAGE(Lobj), "gtk-yes", GTK_ICON_SIZE_MENU);
     gtk_widget_set_sensitive(GTK_WIDGET(Lobjentry), TRUE);
     gtk_label_set_text(GTK_LABEL(Lobjlbl), (gchar *)Avalue);
+
+    if (!strcmp((gchar *)Akey, "mkisofs") && FALSE == conf_get_boolean("mkisofs_support_isov2")) {
+      /* mkisofs est bien la, mais pas de support ISO v2 */
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), get_image("WARNIcon"));
+      g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support ISO 9660 version 2, this function will be disabled."), (gchar *)Akey);
+    } else if (!strcmp((gchar *)Akey, "sox") &&
+        (FALSE == conf_get_boolean("sox_support_mp3") || FALSE == conf_get_boolean("sox_support_ogg"))) {
+      /* sox est bien la, mais pas de support MP3 ou/et OGG */
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), get_image("WARNIcon"));
+      
+      if (FALSE == conf_get_boolean("sox_support_mp3") && FALSE == conf_get_boolean("sox_support_ogg")) {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support neither OGG or MP3 file format, those functions will be disabled."), (gchar *)Akey);
+      } else if (FALSE == conf_get_boolean("sox_support_mp3")) {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support MP3 file format, this function will be disabled."), (gchar *)Akey);
+      } else {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support OGG file format, this function will be disabled."), (gchar *)Akey);
+      }
+    } else {
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), get_image("OKIcon"));
+      g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found."), (gchar *)Akey);
+    }
+    gtk_tooltips_set_tip(sc_grave_get_data(Lg, "_tooltips"), Lobjtips, Ltipstxt, Ltipstxt);
   } else {
-    gtk_image_set_from_stock(GTK_IMAGE(Lobj), "gtk-no", GTK_ICON_SIZE_MENU);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), get_image("BADIcon"));
     gtk_widget_set_sensitive(GTK_WIDGET(Lobjentry), FALSE);
     gtk_label_set_text(GTK_LABEL(Lobjlbl), (gchar *)Akey);
+    g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was not found, some features of graveman will be disabled."), (gchar *)Akey);
+    gtk_tooltips_set_tip(sc_grave_get_data(Lg, "_tooltips"), Lobjtips, Ltipstxt, Ltipstxt);
+
   }
 }
 
-void maj_proprietes_prog(GHashTable *Ahash)
+/* mise a jour de la page "programme externe" */
+void maj_proprietes_prog(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   gchar *Lvalue;
@@ -628,7 +838,7 @@ void maj_proprietes_prog(GHashTable *Ahash)
   for (Lcurentry = Glisteprogrequis; Lcurentry->entry; Lcurentry++) {
     if (Lcurentry->type == CONF_PROG) {
       Lvalue = conf_get_string(Lcurentry->entry);
-      maj_foreach_prog(Lcurentry->entry, Lvalue, Ahash);
+      maj_foreach_prog(Lcurentry->entry, Lvalue, Ag);
     }
   }
 }
@@ -648,7 +858,7 @@ void restore_main_window_pos(GtkWidget *Awindow1)
 }
 
 /* on restaure les dernieres valeurs utilisees pour certains champs */
-void restore_last_used_values(GHashTable *Ahash)
+void restore_last_used_values(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   GtkWidget *Lwidget;
@@ -657,7 +867,7 @@ void restore_last_used_values(GHashTable *Ahash)
     if (!(Lcurentry->level & CONF_SAVEONEXIT)) continue;
 
     if ((Lcurentry->level & CONF_SPIN) || (Lcurentry->level & CONF_CHECK)) {
-      Lwidget = g_hash_table_lookup(Ahash, Lcurentry->entry);
+      Lwidget = sc_grave_get_widget(Ag, Lcurentry->entry);
       if (!GTK_IS_WIDGET(Lwidget)) {
         g_warning("Error %s is missing!", Lcurentry->entry);
         continue;
@@ -688,6 +898,22 @@ void save_last_dir(GtkFileChooser *Awidget)
   if (Llastdir) conf_store_value("lastdir", Llastdir);
 }
 
+/* test sur la version du fichier de configuration .. */
+gboolean test_version()
+{
+  char *Lver = conf_get_string("confversion");
+  if (!Lver || (strcmp(Lver, VERSION) && strcmp(Lver, "0.3.10"))) {
+    gchar *Lerrmsg = g_strdup_printf(_("graveman update !\n\nYou must remove your old configuration file '%s'.\nThen you can restart graveman..\n\nThanks ! :-)"), Gfileconf);
+    GtkWidget *Lerr = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
+                          GTK_BUTTONS_OK, Lerrmsg);
+    gtk_dialog_run(GTK_DIALOG(Lerr));
+    gtk_widget_destroy(Lerr);
+   
+    return FALSE;
+  }
+
+  return TRUE;
+}
 /*
  * vim:et:ts=8:sts=2:sw=2
  */
