@@ -51,6 +51,16 @@ gchar *belleHeure(gchar *Abuf, gchar *Adst)
   return Adst;
 }
 
+gchar *sc_chomp(gchar *Astr)
+{
+  gchar *s;
+  
+  if ((s=strrchr(Astr, 13))) *s=0;
+  if ((s=strrchr(Astr, 10))) *s=0;
+
+  return Astr;
+}
+
 gint sc_strcountv(gchar **Aarray)
 {
   gint i = 0;
@@ -84,12 +94,12 @@ gchar *sc_realloc_cat(gchar *Asep, gchar *Asrc, gchar *Aplus)
 }
 
 /* retourne la valeur d'un combo */
-gchar *get_combo_value(GtkWidget *Acombo)
+gpointer get_combo_value(GtkWidget *Acombo)
 {
   GtkTreeModel *Lmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(Acombo));
   GtkTreeIter Liter;
-  gchar *Lvalue = NULL;
-  if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(Acombo), &Liter)) return g_strdup("");
+  gpointer *Lvalue = NULL;
+  if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(Acombo), &Liter)) return NULL;
 
   gtk_tree_model_get(Lmodel, &Liter, 0, &Lvalue, -1);
 
@@ -109,6 +119,19 @@ gint get_combo_intvalue(GtkWidget *Acombo)
   return Lvalue;
 }
 
+/* retourne la valeur d'un combo */
+gchar *get_combo_value_pos(GtkWidget *Acombo, gint Apos)
+{
+  GtkTreeModel *Lmodel = gtk_combo_box_get_model(GTK_COMBO_BOX(Acombo));
+  GtkTreeIter Liter;
+  gchar *Lvalue = NULL;
+  if (!gtk_combo_box_get_active_iter(GTK_COMBO_BOX(Acombo), &Liter)) return g_strdup("");
+
+  gtk_tree_model_get(Lmodel, &Liter, Apos, &Lvalue, -1);
+
+  return Lvalue;
+}
+
 
 gint make_int(gchar *Abuf)
 {
@@ -171,21 +194,6 @@ gboolean sc_str_has_casesuffix(gchar *Astr, gchar *Asuffix)
   return TRUE;
 }
 
-gpointer sc_hash_table_lookupv(GHashTable *Ahash, gchar *Akeyformat, ...)
-{
-  va_list Largs;
-  gchar *Lkey;
-  gpointer Lreturn;
-
-  va_start(Largs, Akeyformat);
-  Lkey = g_strdup_vprintf(Akeyformat, Largs);
-  va_end(Largs);
-  Lreturn = g_hash_table_lookup(Ahash, Lkey);
-  
-  g_free(Lkey);
-  return Lreturn;
-}
-
 GtkWidget *sc_glade_xml_get_widgetv(GladeXML *Axml, const gchar *Akeyformat, ...) 
 {
   va_list Largs;
@@ -275,56 +283,60 @@ gchar * sc_strescape (const gchar *source)
 
 
   while (*p)
+  {
+    switch (*p)
     {
-      switch (*p)
-      {
-	    case '\b':
-	      *q++ = '\\';
-	      *q++ = 'b';
-	      break;
-	    case '\f':
-	      *q++ = '\\';
-	      *q++ = 'f';
-	      break;
-	    case '\n':
-	      *q++ = '\\';
-	      *q++ = 'n';
-	      break;
-	    case '\r':
-	      *q++ = '\\';
-	      *q++ = 'r';
-	      break;
-	    case '\t':
-	      *q++ = '\\';
-	      *q++ = 't';
-	      break;
-	    case '\\':
-	      *q++ = '\\';
-	      *q++ = '\\';
-	      break;
-	    case '"':
-	      *q++ = '\\';
-	      *q++ = '"';
-	      break;
-	    default:
-		*q++ = *p;
-	      break;
-	    }
-      p++;
-    }
+      case '\b':
+        *q++ = '\\';
+        *q++ = 'b';
+        break;
+      case '\f':
+        *q++ = '\\';
+        *q++ = 'f';
+        break;
+      case '\n':
+        *q++ = '\\';
+        *q++ = 'n';
+        break;
+      case '\r':
+        *q++ = '\\';
+        *q++ = 'r';
+        break;
+      case '\t':
+        *q++ = '\\';
+        *q++ = 't';
+        break;
+      case '\\':
+        *q++ = '\\';
+        *q++ = '\\';
+        break;
+      case '"':
+        *q++ = '\\';
+        *q++ = '"';
+        break;
+      default:
+        *q++ = *p;
+        break;
+      }
+    p++;
+  }
   *q = 0;
   return dest;
 }
 
 /* fin d'un programme externe, on defini une erreur si il ne s'est pas termin correctement */
-void exit_prog(gint Apid, GError **Aerror, gchar *Adefmsg)
+void exit_prog(gint Apid, gboolean Aabort, GError **Aerror, gchar *Adefmsg)
 {
   gint Lcmdstatus = 0;
 
-  kill(Apid, SIGTERM);
+  if (Aabort) {
+    kill(Apid, SIGTERM);
+    sleep(1);
+    kill(Apid, SIGTERM);
+  }
   waitpid(Apid, &Lcmdstatus, 0);
-
   if ((WIFEXITED(Lcmdstatus) && WEXITSTATUS(Lcmdstatus)!=0) || (!(WIFEXITED(Lcmdstatus)))) {
+    _DEB("oui une erreur [%d]\n", Lcmdstatus);
     if (Aerror && !*Aerror) {
       g_set_error(Aerror, GRAVEMAN_ERROR, _ERR_UNKNOWN_ERROR, Adefmsg ? Adefmsg : _("Operation failed"));
     }
@@ -368,6 +380,113 @@ gint sc_mkdir(const gchar *Aname, mode_t Amode) {
   return Lstatus;
 }
 
+/* ne retourne que les numeriques d'une chaine */
+gboolean get_only_numerics(gchar *Abuffer, gint Abufsize, gchar *Asource, gchar Aend)
+{
+  gchar *s = Asource;
+  gchar *d = Abuffer;
+  gint Lnum = 0;
+
+  if (Abufsize<2) return FALSE;
+ 
+  *d=0;
+  while (*s && *s != Aend) {
+    if (isdigit(*s)) {
+      if (++Lnum == Abufsize-1) {
+        *Abuffer = 0;
+        return FALSE;
+      }
+      *(d++)=*s;
+    }
+    s++;
+  }
+  *d=0;
+  
+  return TRUE;
+}
+
+/* retourne une chaine MM:SS en secondes */
+glong get_2time(gchar *Asource)
+{
+  gchar *s = Asource;
+  gchar *e;
+  glong Lret = 0;
+
+  while (*s && !isdigit(*s)) s++;
+  if (!*s) return -1;
+  e=s;
+
+  /* min */
+  while (isdigit(*s)) s++;
+  *(s++)=0;
+  Lret = atol(e) * 60;
+
+  /* sec */
+  while (*s && !isdigit(*s)) s++;
+  if (!*s) return -1;
+  e=s;
+  while (isdigit(*s)) s++;
+  *(s++)=0;
+  Lret += atol(e);
+  
+  return Lret;
+}
+
+gdouble get_pourcent(gchar *Atotal, gchar *Aactu)
+{
+  glong Lnumtotal = atol(Atotal);
+  glong Lnumactu = atol(Aactu);
+
+  if (Lnumtotal == 0) return -1;
+
+  return (1.0/Lnumtotal) * Lnumactu;
+}
+
+gchar *next_no_numeric(gchar *Ade)
+{
+  gchar *Lret = Ade;
+
+  while (*Lret && isdigit(*Lret)) Lret++;
+
+  return *Lret ? Lret : NULL;
+}
+
+gchar *next_putzero(gchar *Asource, gchar Asrc)
+{
+  gchar *s;
+  if ((s=strchr(Asource, Asrc))) {
+    *(s++)=0;
+    return s;
+  }
+
+  return NULL;
+}
+
+gboolean read_all_output_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
+{
+  GIOStatus Lstatus;
+  gchar *Lbuffer = NULL;
+  Tgrave *Lg = (Tgrave *) Adata;
+  GString *Lstrbuf = (GString *) sc_grave_get_data(Lg, "buffer");
+  gint *Lloccont = (gint *) sc_grave_get_data(Lg, "loccont");
+
+  if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
+    *Lloccont = 0;
+    return FALSE;
+  }
+
+  Lstatus = g_io_channel_read_line(Astd, &Lbuffer, NULL, NULL, NULL);  
+  if (!Lbuffer) {
+    return TRUE;
+  }
+
+  Lstrbuf = g_string_append(Lstrbuf, Lbuffer);
+
+  g_free(Lbuffer);
+
+  return TRUE;
+}
+
 /*
  * vim:et:ts=8:sts=2:sw=2
  */
